from enum import Enum
from dataclasses import dataclass
import re

from bs4 import BeautifulSoup
import requests

from akibapass_downloader.const import BASE_URL
from akibapass_downloader.download import Download


class Quality(Enum):
    SD_240P = "240p"
    SD_360P = "360p"
    SD_540P = "540p"
    HD_720P = "720p"
    HD_1080P = "1080p"
    UHD_1440P = "1440p"


@dataclass
class Episode:
    # The URL of the episode.
    url: str

    # The title of the episode.
    name: str

    # The episode numer.
    episode_nr: int

    def get_downloads(
        self, cookies: dict[str, str], filter_quality: Quality | None = None
    ) -> list[Download]:
        """Requests the page and fetches the download links for the episode. @cookies are used for cookies."""
        r = requests.get(
            self.url,
            cookies=cookies,
            headers={
                "Referrer": self.url,
            },
        )
        assert r.status_code == 200

        soup = BeautifulSoup(r.text, "html.parser")

        # Find the episode title
        title = soup.find_all("h1", class_="video-title")[0].strong.string

        # Find the different downloads
        dropdown = soup.find_all("div", class_="dropdown-list")[0]
        scroll = dropdown.ul
        assert scroll.attrs["class"] == ["scrolling"]
        downloads = []
        for download in scroll.find_all("li"):
            a = download.a

            quality = (
                a.string.strip()
                .split("—")[0]
                .rstrip()
                .replace("SD ", "")
                .replace("HD ", "")
            )
            if filter_quality is not None:
                if quality != filter_quality.value:
                    continue

            downloads.append(
                Download(
                    url=f"{BASE_URL}{a.attrs['href']}",
                    quality=quality,
                    filename=f"{title} ({quality}).mp4",
                )
            )

        return downloads


def list_episodes(base_url: str) -> list[Episode]:
    """Fetches episodes for @base_url."""
    r = requests.get(base_url)
    assert r.status_code == 200

    soup = BeautifulSoup(r.text, "html.parser")

    episode_nr_matcher = re.compile(r".*S[0-9]+E([0-9]+).*")
    episodes = []
    for li in soup.find_all("li", class_="item-type-video"):
        url = li.find_all("a", class_="browse-item-link")[0]
        img = url.find_all("img")[0]
        name_raw = img.attrs["alt"]
        match = episode_nr_matcher.match(name_raw)

        episodes.append(
            Episode(
                url=f"{BASE_URL}{url.attrs['href']}",
                name=name_raw,
                episode_nr=int(match.group(1)),
            )
        )
    return episodes
